/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: PrintCM.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "tools.h"
#include "memory.h"

void PrintCMp(FILE *f, double **ar, double **ai, int n, int m, char *s)
{
  int i, j;
  double amp, mod;

  fprintf(f, " %s:\n", s);
  fprintf(f, "  |");
  for(j=0; j<m; ++j)fprintf(f, " %15d", j);
  fprintf(f, "\n");
  for(i=0; i<n; ++i) {
    fprintf(f, "%2d|", i);
    for(j=0; j<m; ++j) {
      amp = sqrt(ar[i][j] * ar[i][j] + ai[i][j] * ai[i][j]);
      if(amp > 1.e-10)
      {
	mod = atan2(ai[i][j], ar[i][j]);
	if(fabs(mod - M_PI) < 1.e-10 ||
	   fabs(mod + M_PI) < 1.e-10)
	  {
	    mod = 0.0;
	    amp *= -1.0;
	  }
	if(fabs(mod) < 1.e-10)
	  fprintf(f, " %+9.3g@(%4s)", amp, ".");
	else
	  fprintf(f, " %+9.3g@(%+4.2f)", amp, mod / M_PI);
      }
      else
	fprintf(f, " %9s(%4s)", ".", ".");
    }
    fprintf(f, "\n");
  }
}

void PrintCM(FILE *f, double *ar, double *ai, int n, int m, char *s)
{
  int i;
  double **arp, **aip;

  arp = dpmalloc(n);
  aip = dpmalloc(n);
  for(i=0; i<n; ++i) {
    arp[i] = ar + i*m;
    aip[i] = ai + i*m;
  }
  PrintCMp(f, arp, aip, n, m, s);
  myfree(arp);
  myfree(aip);
}

void PrintCMPart(FILE *f, double *ar, double *ai, int n, int m, char *s, 
	int nlo, int nhi, int mlo, int mhi)
{
  int i, j, nn = nhi-nlo+1, mm = mhi-mlo+1;
  double *br, *bi;
  br = dmalloc(nn * mm);
  bi = dmalloc(nn * mm);
  for(i=nlo; i<=nhi; ++i)
    for(j=mlo; j<=mhi; ++j) {
      br[(i-nlo)*mm + (j-mlo)] = ar[i*m + j];
      bi[(i-nlo)*mm + (j-mlo)] = ai[i*m + j];
    }
  PrintCM(f, br, bi, nn, mm, s);
  myfree(br);
  myfree(bi);
}
