/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: PrintRM.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include "tools.h"
#include "memory.h"

void PrintRMp(FILE *f, double **a, int n, int m, char *s)
{
  int i, j;

  fprintf(f, " %s:\n", s);
  fprintf(f, "  |");
  for(j=0; j<m; ++j)fprintf(f, " %9d", j);
  fprintf(f, "\n");
  for(i=0; i<n; ++i) {
    fprintf(f, "%2d|", i);
    for(j=0; j<m; ++j)fprintf(f, " %+9.3g", a[i][j]);
    fprintf(f, "\n");
  }
}

void PrintRM(FILE *f, double *a, int n, int m, char *s)
{
  int i;
  double **ap;

  ap = dpmalloc(n);
  for(i=0; i<n; ++i) ap[i] = a + i*m;
  PrintRMp(f, ap, n, m, s);
  myfree(ap);
}

void PrintRMPart(FILE *f, double *a, int n, int m, char *s, 
	int nlo, int nhi, int mlo, int mhi)
{
  int i, j, nn = nhi-nlo+1, mm = mhi-mlo+1;
  double *b;
  b = dmalloc(nn * mm);
  for(i=nlo; i<=nhi; ++i)
    for(j=mlo; j<=mhi; ++j)
      b[(i-nlo)*mm + (j-mlo)] = a[i*m + j];
  PrintRM(f, b, nn, mm, s);
  myfree(b);
}
