/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: dmdtimer.h,v 1.3 1995/11/29 22:15:43 deaven Exp deaven $
 * $Author: deaven $
 */

#ifndef _DMDTIMER_H_
#define _DMDTIMER_H_
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Some (most) systems have very fast system calls that get
 * the current clock (the zero of which we don't care).
 * On these, we can use a macro expansion for MyClock() which
 * evaluates to a time in seconds.  Otherwise, we'll use the 
 * BSD getrusage call or some other system call, which might be
 * very slow.
 *
 * Note: in NetBSD, clock() just calls getrusage!
 */
#if   defined(__I860__) && !defined(SUNMOS)
#include <nx.h>
#define MyClock()	_dclock()

#elif defined(_AIX)
#include <time.h>
#define MyClock()	(clock() * (1.0 / CLOCKS_PER_SEC))

#else
double MyClock(void);

#endif
double WallClock(void);

#ifdef ENABLETIMER

#define ENTER(name)  timerenter(name, __FILE__, __LINE__)
#define LEAVE()  timerleave(__FILE__, __LINE__)
#define REPORT(file) timerreport(file)

#else

#define ENTER(name) ;
#define LEAVE() ;
#define REPORT(file) ;

#endif /* ENABLETIMER */

typedef struct  {
  double tuser;		/* user time in seconds */
  double tsystem;	/* system time in seconds */
  long faults;		/* number of page faults */
} ResourceInfo;

void GetResources(ResourceInfo *ri);

void timerenter(char *id, char *file, int line);
void timerleave(char *file, int line);
void timerreport(FILE *f);

#ifdef __cplusplus
}
#endif

#endif /*  _DMDTIMER_H_ */
