/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: fit.h,v 1.2 1996/05/29 21:11:16 deaven Exp deaven $
 * $Author: deaven $
 */

#ifndef _DMD_FIT_H_
#define _DMD_FIT_H_

#include <stdio.h>
#include <float.h>

#ifdef __cplusplus
extern "C" {
#endif

#define NO_RESTRICTION	DBL_MAX	

/* Structure used to define a fitting problem, and pass to the
 * fitting package.
 */
typedef struct {
	/* information about the parameters */
	int npar;		/* number of parameters */
	int *fitpar;		/* which parameters to fit */
	char **parname;		/* parameter names */
	double *par;		/* current values */
	double *parmin;		/* minimum value (may be NO_RESTRICTION) */
	double *parmax;		/* maximum value (may be NO_RESTRICTION) */
	int changed;		/* must be set to non-zero if changes */

	/* information about the experiments */
	int nexpt;		/* number of `experiments' */
	double *exptweight;	/* relative weight of this data */
	char **exptname;	/* experiment names */
	double *exptdata;	/* array of experimental values */

	/* methods for evaluating the model */
	double *expt;		/* last evaluated model */
	double rms;		/* last evaluated rms value */
	int (*model)(double *par, double *expt, void *res);
				/* evaluate the model */
	void *resources;	/* anything the model requires */
} *Fit;

Fit NewFit(int npar, int nexpt);
void DestroyFit(Fit fit);
void FitPrint(FILE *f, Fit fit);
int FitEvaluate(Fit fit);
double FitTrialRange(Fit fit, int which);

int FitMain(Fit fit, int argc, char **argv);

int FitByConjugateGradient(Fit fit, char *options, FILE *fout);
int FitByDownhillSimplex(Fit fit, char *options, FILE *fout);
int FitByGridSearch(Fit fit, char *options, FILE *fout);
int FitByMonteCarlo(Fit fit, char *options, FILE *fout);

#ifdef __cplusplus
}
#endif

#endif /* _DMD_FIT_H_ */

