/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: fit_mc.c,v 1.2 1996/05/29 21:11:16 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "memory.h"
#include "tools.h"
#include "random.h"
#include "fit.h"

struct resources {
	int iter;		/* number of iterations */
	double temp;		/* temperature */
};

/* Monte Carlo fitting.
 */
int FitByMonteCarlo(Fit fit, char *options, FILE *fout)
{
	int i, r = 0;
	double *plast, rmslast = 0.0;
	double *pbest, rmsbest = 0.0;
	int np, *which, it;

	struct resources res;

	res.iter = 512;
	res.temp = 0.01;

	/* Get a list of parameters to be fit */
	which = imalloc(fit->npar);
	for(i=np=0; i<fit->npar; ++i)
	  if(fit->fitpar[i]) which[np++] = i;
	if(np < 1) {myfree(which); return 0;}

	plast = dmalloc(np);
	pbest = dmalloc(np);

	for(it = 0; it < res.iter; ++it)
	{
	  /* Choose a random step.
	   */
	  if(it > 0) 
	  {
	    for(i=0; i<np; ++i)
	    {
	      fit->par[which[i]] +=
		(ran3() - 0.5) * FitTrialRange(fit, which[i]);
	    }
	    fit->changed = 1;
	  }

	  /* Evaluate the new fitness.
	   */
	  if((r = FitEvaluate(fit)) != 0) break;

	  /* Is it a record-breaker?
	   */
	  if(it == 0 || fit->rms < rmsbest)
	  {
	    for(i=0; i<np; ++i)
	      pbest[i] = fit->par[which[i]];
	    rmsbest = fit->rms;
	  }

	  /* Accept this new point?
	   */
	  if(it == 0 || exp((rmslast - fit->rms) / res.temp) > ran3())
	  {
	    rmslast = fit->rms;
	    for(i=0; i<np; ++i)
	      plast[i] = fit->par[which[i]];
	  }
	  else
	  {
	    for(i=0; i<np; ++i)
	      fit->par[which[i]] = plast[i];
	    fit->changed = 1;
	  }

	}

	/* put back the best data set */
	for(i=0; i<np; ++i)
	  fit->par[which[i]] = pbest[i];
	fit->changed = 1;

	myfree(plast);
	myfree(pbest);
	myfree(which);

	return r;
}

