/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gcontract.c,v 1.3 1996/05/03 19:41:26 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include "geometry.h"
#include "neighbor.h"

/* Locate atoms in a geometry which are too far away from any other
 * atom and move them closer to their nearest neighbor, until all
 * atoms have a valid nearest neighbor.
 */
Geometry *Gcontract(Geometry *g, double dmax)
{
  int i, j, n, m, nn[3];
  double d, x[3], dx[3], eps = dmax*0.0001;
  struct nl_s *nl;

  nl = nlalloc(dmax);

  do {
    GCentralCell(g);
    newnl(g, nl);
    n = 0;
    for(i=0; i<(g->n); ++i) {
      if(nl->mrkr[i] == nl->mrkr[i+1])
      {
	j = nearnb(g, i, &d, dx, nn);
	(void) mmul3(g->box, g->x+3*i, x);
	for(m=0; m<3; ++m) x[m] += dx[m] * (d - dmax + eps);
	(void) mmul3(g->boxi, x, g->x+3*i);
        ++n;
	break;
      }
    }
  } while(n > 0);

  nlfree(nl);
  return g;
}
