/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gcpy.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include "geometry.h"
#include "memory.h"

/* Copy one geometry to another.
 */
Geometry *Gcpy(Geometry *dst, Geometry *src)
{
	int i;

	dst = Grealloc(dst, src->n);

	dst->period = src->period;

	for(i=0; i<9; ++i) {
	  dst->box[i] = src->box[i];
	  dst->boxi[i] = src->boxi[i];
	}

	if(src->flags != NULL) {
	  dst->flags = myrealloc(dst->flags, dst->n * sizeof(int));
	  for(i=0; i<(dst->n); ++i) dst->flags[i] = src->flags[i];
	} else {
	  myfree(dst->flags);
	  dst->flags = NULL;
	}

	for(i=0; i<(3 * dst->n); ++i) dst->x[i] = src->x[i];

	for(i=0; i<dst->n; ++i) {
	  myfree(dst->aname[i]);
	  dst->aname[i] = mystrdup(src->aname[i]);
	}

	myfree(dst->name);
	dst->name = mystrdup(src->name);

	return dst;
}

