/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gdigitize.c,v 1.4 1996/02/07 16:49:25 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include "geometry.h"
#include "memory.h"

/* Convert a geometry into a linear binary representation.
 * Allocate new space for the buffer, and also return its size.
 */
unsigned char *Gdigitize(Geometry *g, unsigned long *size)
{
  unsigned int i, j;
  int m;
  unsigned long sz;
  unsigned char *buffer;
  sz = sizeof(int) * 2 + sizeof(double) * (18 + 3*(g->n));
  sz += sizeof(char *);
  if(g->name != NULL)
    sz += strlen(g->name) + 1;
  for(m=0; m<(g->n); ++m)
  {
    sz += sizeof(char *);
    if(g->aname[m] != NULL)
      sz += strlen(g->aname[m]) + 1;
  }
  sz += sizeof(int *);
  if(g->flags != NULL)
    sz += g->n * sizeof(int);

  buffer = mymalloc(sz);
  memcpy(buffer, &(g->n), i = sizeof(int));
  memcpy(buffer + i, g->x, j = 3 * g->n * sizeof(double)); i += j;
  memcpy(buffer + i, g->box, j = 9 * sizeof(double)); i += j;
  memcpy(buffer + i, g->boxi, j = 9 * sizeof(double)); i += j;
  memcpy(buffer + i, &g->period, j = sizeof(int)); i += j;
  memcpy(buffer + i, &g->name, j = sizeof(char *)); i += j;
  if(g->name != NULL)
  {
    memcpy(buffer + i, g->name, j = strlen(g->name) + 1); i += j;
  }
  for(m=0; m<(g->n); ++m)
  {
    memcpy(buffer + i, g->aname + m, j = sizeof(char *)); i += j;
    if(g->aname[m] != NULL)
    {
      memcpy(buffer + i, g->aname[m], j = strlen(g->aname[m]) + 1); i += j;
    }
  }
  memcpy(buffer + i, &(g->flags), j = sizeof(int *)); i += j;
  if(g->flags != NULL)
  {
    memcpy(buffer + i, g->flags, j = g->n * sizeof(int)); i += j;
  }

  if(i != sz) fprintf(stderr, "Gdigitize: i=%d != size=%ld\n", i, sz);
  *size = sz;
  return buffer;
}

/* Unpack a geometry that was saved with Gdigitize() in a
 * binary format.
 */
Geometry *Gundigitize(unsigned char *buffer, unsigned long size)
{
  int n, m;
  unsigned int i, j;
  Geometry *g;

  memcpy(&n, buffer, i = sizeof(int));
  g = Gmalloc(n, NULL);
  memcpy(g->x, buffer + i, j = 3 * g->n * sizeof(double)); i += j;
  memcpy(g->box, buffer + i, j = 9 * sizeof(double)); i += j;
  memcpy(g->boxi, buffer + i, j = 9 * sizeof(double)); i += j;
  memcpy(&g->period, buffer + i, j = sizeof(int)); i += j;
  memcpy(&g->name, buffer + i, j = sizeof(char *)); i += j;
  if(g->name != NULL)
  {
    g->name = mystrdup((char *) buffer + i); i += strlen(g->name) + 1;
  }
  for(m=0; m<(g->n); ++m)
  {
    memcpy(g->aname + m, buffer + i, j = sizeof(char *)); i += j;
    if(g->aname[m] != NULL)
    {
      g->aname[m] = mystrdup((char *) buffer + i); i += strlen(g->aname[m]) + 1;
    }
  }
  memcpy(&(g->flags), buffer + i, j = sizeof(int *)); i += j;
  if(g->flags != NULL)
  {
    g->flags = imalloc(g->n);
    memcpy(g->flags, buffer + i, j = g->n * sizeof(int)); i += j;
  }
  if(i != size) fprintf(stderr, "Gundigitize: i=%d != size=%ld\n", i, size);
  return g;
}
