/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gdigitize2.c,v 1.5 1996/03/19 18:42:51 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <string.h>
#ifdef sgi
#include <bstring.h>
#endif
#include <math.h>
#include "geometry.h"
#include "memory.h"
#include "ansiconvert.h"

#define ADD_INT(foo) \
  { intansi(buffer + i, foo); i += ANSI_INT_SIZE; }
#define ADD_DOUBLE(foo) \
  { doubleansi(buffer + i, foo); i += ANSI_DOUBLE_SIZE; }
#define ADD_STRING(foo) \
  { if(foo == NULL) buffer[i++] = 0; \
    else { \
      bcopy(foo, buffer + i, strlen(foo) + 1); \
      i += strlen(foo) + 1; \
    } \
  }

#define GET_INT(foo) \
  { foo = ansiint(buffer + i); i += ANSI_INT_SIZE; }
#define GET_DOUBLE(foo) \
  { foo = ansidouble(buffer + i); i += ANSI_DOUBLE_SIZE; }
#define GET_STRING(foo) \
  { foo = mystrdup(((char *) buffer) + i); i += strlen(foo) + 1; }

/* Convert a geometry into a linear binary representation.
 * Use the provided buffer starting at position offset; reallocate the
 * buffer if it is too small.
 * The size is also stored as an integer at the beginning of
 * the buffer + offset.
 * Advance offset to point to the next byte that is not part of
 * the geometry data.
 */
unsigned char *Gdigitize_buf_ansi(Geometry *g, unsigned char *buffer,
	unsigned long *offset, unsigned long *size)
{
	unsigned int i, j;
	int one = 1, zero = 0;
	unsigned long sz;

	sz = ANSI_INT_SIZE * 4 + ANSI_DOUBLE_SIZE * (18 + 3*(g->n));
	sz += (g->name != NULL ? strlen(g->name) : 0) + 1;
	for(j=0; j<(g->n); ++j)
	    sz += (g->aname[j] != NULL ? strlen(g->aname[j]) : 0) + 1;
	if(g->flags != NULL)
	  sz += g->n * ANSI_INT_SIZE;

	if((*offset) + sz > *size) {
	  *size = (*offset) + sz;
	  buffer = myrealloc(buffer, *size);
	}

	i = *offset;
	ADD_INT((int) sz);
	ADD_INT(g->n);
	ADD_INT(g->period);
	ADD_INT(g->flags == NULL ? zero : one);

	ADD_STRING(g->name);

	for(j=0; j<9; ++j) ADD_DOUBLE(g->box[j]);
	for(j=0; j<9; ++j) ADD_DOUBLE(g->boxi[j]);

	for(j=0; j<(3*g->n); ++j) ADD_DOUBLE(g->x[j]);

	for(j=0; j<g->n; ++j) ADD_STRING(g->aname[j]);

	if(g->flags != NULL) {
	  for(j=0; j<g->n; ++j) ADD_INT(g->flags[j]);
	}

	if(i != (*offset) + sz)
	  fprintf(stderr,
	    "Gdigitize_ansi: i=%d != size=%ld\n", i, (*offset) + sz);

	(*offset) += sz;

	return buffer;
}

/* Unpack a geometry that was saved with Gdigitize_buf_ansi() in a
 * binary format.  Begin reading the buffer at offset, and adjust
 * offset to point to the next byte that is not part of the geometry
 * data on return.
 */
Geometry *Gundigitize_buf_ansi(unsigned char *buffer, unsigned long *offset,
	unsigned long size)
{
	int n, sz, flags;
	unsigned int i, j;
	Geometry *g;

	i = *offset;
	GET_INT(sz);
	if((*offset) + sz > size) {
	  fprintf(stderr,
	    "Gundigitize_buf_ansi: bad buffer (too small -- %ld, need %ld)\n",
	    size, (*offset) + sz);
	  return NULL;
	}

	GET_INT(n);
	g = Gmalloc(n, NULL);
	GET_INT(g->period);
	GET_INT(flags);
	GET_STRING(g->name);
	for(j=0; j<9; ++j) GET_DOUBLE(g->box[j]);
	for(j=0; j<9; ++j) GET_DOUBLE(g->boxi[j]);
	for(j=0; j<(3*g->n); ++j) GET_DOUBLE(g->x[j]);
	for(j=0; j<g->n; ++j) GET_STRING(g->aname[j]);

	if(flags) {
	  g->flags = imalloc(g->n);
	  for(j=0; j<g->n; ++j) GET_INT(g->flags[j]);
	}

	if(i != (*offset) + sz) 
	  fprintf(stderr, "Gundigitize_buf_ansi: i=%d != size=%ld\n",
	  i, (*offset) + sz);

	(*offset) = i;

	return g;
}

/* Convert a geometry into a linear binary representation.
 * Allocate new space for the buffer, and also return its size.
 * The size id also stored as an integer at the beginning of
 * the buffer.
 */
unsigned char *Gdigitize_ansi(Geometry *g, unsigned long *size)
{
	unsigned char *buffer = NULL;
	unsigned long offset = 0;
	*size = 0;
	return Gdigitize_buf_ansi(g, buffer, &offset, size);
}

/* Unpack a geometry that was saved with Gdigitize_ansi() in a
 * binary format.
 */
Geometry *Gundigitize_ansi(unsigned char *buffer, unsigned long size)
{
	unsigned long offset = 0;
	return Gundigitize_buf_ansi(buffer, &offset, size);
}

