/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: glattice.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <string.h>
#include "geometry.h"
#include "memory.h"

/* Construct a lattice from the geometry (a unit cell).
 * If a direction is not periodic, the lattice is not
 * created in that direction.
 */
Geometry *Glattice(Geometry *g, 
  int nx0, int nx1, int ny0, int ny1, int nz0, int nz1)
{
  int i, x, y, z, n;
  double d[3], p[3], q[3], newbox[3*3];
  Geometry *lg;

  if((g->period & GP_PX) == 0) nx0 = nx1 = 0;
  if((g->period & GP_PY) == 0) ny0 = ny1 = 0;
  if((g->period & GP_PZ) == 0) nz0 = nz1 = 0;

  n = (nx1 - nx0 + 1) * (ny1 - ny0 + 1) * (nz1 - nz0 + 1);
  lg = Grealloc(Gcpy(NULL, g), g->n * n);

  for(i=0; i<3; ++i) {
    newbox[3*i + 0] = lg->box[3*i + 0] * (nx1 - nx0 + 1);
    newbox[3*i + 1] = lg->box[3*i + 1] * (ny1 - ny0 + 1);
    newbox[3*i + 2] = lg->box[3*i + 2] * (nz1 - nz0 + 1);
  }
  Gsetbox(lg, newbox);

  n = 0;
  for(x=nx0; x<=nx1; ++x) {
    d[0] = x;
    for(y=ny0; y<=ny1; ++y) {
      d[1] = y;
      for(z=nz0; z<=nz1; ++z) {
	d[2] = z;
	for(i=0; i<(g->n); ++i) {
	  Gcopyatom(lg, n, g, i);
	  mmul3(g->box, d, p);		/* to physical space */
	  mmul3(lg->boxi, p, q);	/* to lg box space */
	  (lg->x + 3*n)[0] += q[0];
	  (lg->x + 3*n)[1] += q[1];
	  (lg->x + 3*n)[2] += q[2];
	  myfree(lg->aname[n]);
	  lg->aname[n] = mystrdup(g->aname[i]);
	  ++n;
	}
      }
    }
  }

  return lg;
}
