/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gorient.c,v 1.2 1996/02/12 15:32:57 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "memory.h"
#include "geometry.h"

#define RESOLUTION 512
#define TWOPI (2.0 * M_PI)

static int encode(double angle)
{
	angle /= TWOPI;
	angle -= floor(angle); /* for special cases */
	while(angle > 1.0) angle -= 1.0;
	while(angle < 0.0) angle += 1.0;
	angle *= (double) RESOLUTION;
	return ((int) angle);
}

static double decode(int angle)
{
	double r = angle;
	r *= TWOPI / ((double) RESOLUTION);
	return r;
}

/* Encode Yaw Pitch Roll angles.
 */
static int orient_encode(double ypr[3])
{
	return encode(ypr[0]) +
	       encode(ypr[1]) * RESOLUTION +
	       encode(ypr[2]) * (RESOLUTION * RESOLUTION);
}

static void orient_decode(int code, double ypr[3])
{
	int ang;
	ang = code / (RESOLUTION * RESOLUTION);
	ypr[2] = decode(ang);
	code -= ang * (RESOLUTION * RESOLUTION);
	ang = code / RESOLUTION;
	ypr[1] = decode(ang);
	code -= ang * RESOLUTION;
	ypr[0] = decode(code);
}

Geometry *GSetFlagsFromYPR(Geometry *g, double *ypr)
{
        int i;

        if(g->flags == NULL)
          g->flags = imalloc(g->n);

        for(i=0; i<g->n; ++i)
          g->flags[i] = orient_encode(ypr + 3*i);
	
	return g;
}

double *GSetYPRFromFlags(Geometry *g, double *ypr)
{
        int i;

        if(g->flags == NULL)
        {
          g->flags = imalloc(g->n);
          for(i=0; i<g->n; ++i) g->flags[i] = 0;
        }

        for(i=0; i<g->n; ++i)
          orient_decode(g->flags[i], ypr + 3*i);

	return ypr;
}
