/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gprettycell.c,v 1.1 1996/05/07 17:21:30 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include "dmd/geometry.h"
#include "dmd/neighbor.h"
#include "dmd/memory.h"

static int add(Geometry *g, NeighborList *nl, int i, char *isdone)
{
	int jnl, j, k, n[3];
	int ndone = 0;
	double dx[3], dxb[3];

	for(jnl=nl->mrkr[i]; jnl<nl->mrkr[i+1]; ++jnl) {
	  j = getnlindex(nl->list[jnl]);
	  if(j != i && !isdone[j]) {
	    getnlnn(nl->list[jnl], n);
	    getnldx(g, i, j, n, dxb, dx);
	    for(k=0; k<3; ++k) {
	      if(g->period & GP_PDIR(k)) {
		while(dxb[k] > 0.5) dxb[k] -= 1.0;
		while(dxb[k] < -0.5) dxb[k] += 1.0;
	      }
	      g->x[3*j + k] = g->x[3*i + k] - dxb[k];
	    }
	    isdone[j] = 1;
	    ++ndone;
	  }
	}

	return ndone;
}

/* Shift atoms by unit cell distances to make the central cell
 * atoms bonded within the cell as much as possible.
 */
Geometry *GPrettyCell(Geometry *g, double rbond)
{
	int i, j, r, ndone = 0, ndone_in_sweep;
	char *isdone;
	NeighborList *nl = nlalloc(rbond);

	isdone = mymalloc(g->n * sizeof(*isdone));
	for(i=0; i<g->n; ++i) isdone[i] = 0;
	newnl(g, nl);

	while(ndone < g->n) {
	  /* Pick the first unconsidered atom to work on next.
	   */
	  for(i=0; i<g->n && isdone[i]; ++i);

	  /* Mark it as done, and pick all neighboring atoms (within
	   * rbond), and place them in proper relation to atom i.
	   */
	  isdone[i] = 1;
	  ++ndone;

	  do {
	    ndone_in_sweep = 0;
	    for(j=0; j<g->n; ++j) {
	      if(isdone[j]) {
		while((r = add(g, nl, j, isdone)) > 0)
		  ndone_in_sweep += r;
	      }
	    }
	    ndone += ndone_in_sweep;
	  } while(ndone_in_sweep > 0);
	}

	myfree(isdone);
	nlfree(nl);
	return g;
}
