/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gsplit.c,v 1.1 1996/05/07 21:38:09 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "dmd/geometry.h"
#include "dmd/memory.h"

/* Given a geometry g, split it up into a NULL-terminated list 
 * of geometries based on a passed function that indicates whether
 * a test atom belongs to the group or not.
 */
Geometry **GSplit(Geometry *g,
	int (*ismember)(Geometry *group, Geometry *test, int atom))
{
	int i, j, nadded, nresult = 0;
	char *used = mymalloc(g->n * sizeof(*used));
	Geometry **result = mymalloc(sizeof(*result)), *gc;

	result[0] = NULL;
	for(i=0; i<g->n; ++i) used[i] = 0;

	do {
	  /* Add the next unused atom to a new group.
	   */
	  for(i=0; i<g->n && used[i]; ++i);

	  if(i >= g->n) break; /* we are done */

	  gc = Grealloc(Gcpy(NULL, g), 1);
	  Gcopyatom(gc, 0, g, i);
	  used[i] = 1;

	  ++nresult;
	  result = myrealloc(result, (nresult + 1) * sizeof(*result));
	  result[nresult - 1] = gc;
	  result[nresult] = NULL;

	  do {
	    nadded = 0;
	    for(j=0; j<g->n; ++j) {
	      if(!used[j] && ismember(gc, g, j)) {
		Grealloc(gc, gc->n + 1);
		Gcopyatom(gc, gc->n - 1, g, j);
		used[j] = 1;
		++nadded;
	      }
	    }
	  } while(nadded > 0);

	} while(1);

	myfree(used);

	return result;
}
