/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: gstdorient.c,v 1.4 1996/04/19 01:52:51 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "geometry.h"
#include "memory.h"
#include "tools.h"

/* If this geometry is a complete specification, translate to center.
 * If there are periodicity flags, then remove extra box translations.
 * If there are no periodicity flags, rotate to align the principle
 * moments of inertia with the cartesian axes.
 *
 * David Deaven <deaven@ishmael.ameslab.gov>
 */
Geometry *Gstdorient(Geometry *g)
{
  int i, j;
  static double xcam[3], mom[3], axes[9], vcam[3], up[3], t[16];
  if(g->period & GP_INCOMPLETE) return g; /* don't mess with it */
  if(g->period & GP_PERIODIC) /* cannot rotate it */
  {
    for(j=0; j<3; ++j)
    {
      xcam[j] = 0.0;
      vcam[j] = (j==2 ? -1.0 : 0.0);
      up[j] = (j==1 ? 1.0 : 0.0);
      for(i=0; i<g->n; ++i)
      {
	if((g->period >> j) & 1)
	  g->x[3*i + j] -= anint(g->x[3*i +j]);
	xcam[j] += g->x[3*i + j];
      }
      xcam[j] /= (double) g->n;
    }
  }
  else
  {
    Gmoment(g, NULL, mom, xcam, axes);
    for(i=0; i<3; ++i)
    {
      vcam[i] = -axes[i + 6];
      up[i] = axes[i + 3];
    }
  }
  Gentra(xcam, vcam, up, t, axes); /* generate transformation */
  for(i=0; i<(g->n); ++i)
  {
    Transf(mmul3(g->box, g->x+i*3, xcam), t, vcam);
    (void) mmul3(g->boxi, vcam, g->x+i*3);
  }
  return g;
}
