/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: matinv3.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <math.h>
#include "geometry.h"

/* Invert a 3x3 matrix
 */
void matinv3(double aix[3*3], double ax[3*3], double *det)
{
  double *ai[3], *a[3];
  double d00, d01, d10, d20, d21, d02, d11, d12, d22;
  ai[0] = aix; ai[1] = aix+3; ai[2] = aix+6;
  a[0] = ax; a[1] = ax+3; a[2] = ax+6;
  d00=a[1][1]*a[2][2]-a[1][2]*a[2][1];
  d01=a[1][2]*a[2][0]-a[1][0]*a[2][2];
  d10=a[2][1]*a[0][2]-a[0][1]*a[2][2];
  d20=a[0][1]*a[1][2]-a[1][1]*a[0][2];
  d21=a[0][2]*a[1][0]-a[0][0]*a[1][2];
  d02=a[1][0]*a[2][1]-a[2][0]*a[1][1];
  d11=a[0][0]*a[2][2]-a[0][2]*a[2][0];
  d12=a[2][0]*a[0][1]-a[0][0]*a[2][1];
  d22=a[0][0]*a[1][1]-a[0][1]*a[1][0];
  *det =a[0][0]*d00+a[0][1]*d01+a[0][2]*d02;
  if(*det != 0.0) {
    ai[0][0]=d00/(*det);
    ai[1][1]=d11/(*det);
    ai[2][2]=d22/(*det);
    ai[0][1]=d10/(*det);
    ai[0][2]=d20/(*det);
    ai[1][2]=d21/(*det);
    ai[1][0]=d01/(*det);
    ai[2][0]=d02/(*det);
    ai[2][1]=d12/(*det);
  }
}

