/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: memory.c,v 1.7 1996/05/29 21:16:48 deaven Exp $
 * $Author: deaven $
 */

#ifdef TRACKMEM
#undef TRACKMEM
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "memory.h"
#include "tools.h"

static void fatal(char *op, size_t size)
{
	char buf[256];
	sprintf(buf, "pid %d: cannot %s (size = %d)\n",
	  (int) getpid(), op, (int) size);
	dmd_fatal_error(buf);
}

/* Re-allocate a block, or allocate it if NULL pointer is passed.
 */
void *myrealloc(void *ptr, size_t size)
{
	void *r;

	if(ptr == NULL)
	  return mymalloc(size);

	if(size == 0) {
	  myfree(ptr);
	  return NULL;
	}
	if((r = realloc(ptr, size)) == NULL) fatal("myrealloc", size);
	return r;
}

/* Allocate a block.
 */
void *mymalloc(size_t size)
{
	void *r;
	if(size == 0) return NULL;
	r = malloc(size);
	if(r == NULL) fatal("mymalloc", size);
	return r;
}

/* Free a memory block 
 */
void myfree(void *ptr)
{
	if(ptr != NULL) free(ptr);
}

/* A replacement for strdup(), which is not always present.
 */
char *mystrdup(char *s)
{
	int n = 0;
	char *copy;
	if(s==NULL) return NULL;
	while(s[n] != '\0') ++n;
	copy = mymalloc((n+1) * sizeof(char));
	for(; n>=0; --n) copy[n] = s[n];
	return copy;
}

