/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: mmul3.c,v 1.5 1996/08/08 19:48:17 deaven Exp $
 * $Author: deaven $
 */

#include "tools.h"

/* These functions can be coded in assembly for extra speed on
 * a given machine, or implemented as macros.
 */

/* 3x3 matrix -- vector multiply
 * x[] and y[] must be different!
 */
double *mmul3(double a[3*3], double x[3], double y[3])
{
	register int i;

	for(i=0; i<3; i++) {
	  y[i] = a[i*3]*x[0] + a[i*3 + 1]*x[1] + a[i*3 + 2]*x[2];
	}

	return y;
}

/* 3x3 matrix -- vector multiply
 */
double *mtrn3(double a[3*3], double x[3])
{
	register int i;
	double y[3];

	for(i=0; i<3; i++) {
	  y[i] = a[i*3]*x[0] + a[i*3 + 1]*x[1] + a[i*3 + 2]*x[2];
	}
	for(i=0; i<3; i++)
	  x[i] = y[i];

	return x;
}

/* 3x3 matrix multiply on a vector.
 */
double *vmmul3(double a[3*3], double *x, double *y, int nvec)
{
	register int i;
	if(x == y) {
	  for(i=0; i<nvec; i++)
	    mtrn3(a, x + 3*i);
	} else {
	  for(i=0; i<nvec; i++)
	    mmul3(a, x + 3*i, y + 3*i);
	}

	return y;
}

/* 3x3 matrix -- vector multiply by transpose
 * x[] and y[] must be different!
 */
double *mtmul3(double a[3*3], double x[3], double y[3])
{
	register int i;

	for(i=0; i<3; i++) {
	  y[i] = a[i]*x[0] + a[3 + i]*x[1] + a[6 + i]*x[2];
	}

	return y;
}

/* 3x3 matrix -- vector multiply by transpose
 */
double *mttrn3(double a[3*3], double x[3])
{
	register int i;
	double y[3];

	for(i=0; i<3; i++) {
	  y[i] = a[i]*x[0] + a[3 + i]*x[1] + a[6 + i]*x[2];
	}
	for(i=0; i<3; i++)
	  x[i] = y[i];

	return x;
}

/* 3x3 matrix multiply on a vector.
 */
double *vmtmul3(double a[3*3], double *x, double *y, int nvec)
{
	register int i;
	if(x == y) {
	  for(i=0; i<nvec; i++)
	    mttrn3(a, x + 3*i);
	} else {
	  for(i=0; i<nvec; i++)
	    mtmul3(a, x + 3*i, y + 3*i);
	}

	return y;
}

