/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: neighbor.c,v 1.3 1996/05/03 18:31:44 deaven Exp $
 * $Author: deaven $
 */

/*==========================================================================
 * MD neighbor list routines
 *
 * David Deaven 24 February 1994 deaven@ishmael.ameslab.gov 
 *========================================================================*/
#include <stdio.h>
#include <math.h>
#include "neighbor.h"
#include "memory.h"

#ifdef _I860_
#define sqrt(x) __builtin_sqrt(x)
#endif

/*==========================================================================
 * free allocated neighbor list.
 *========================================================================*/
void nlfree(NeighborList *nl)
{
  if(nl != NULL) {
    if(nl->list != NULL)myfree(nl->list);
    if(nl->mrkr != NULL)myfree(nl->mrkr);
    myfree(nl);
  }
}

/*==========================================================================
 * allocate neighbor list.
 *========================================================================*/
NeighborList *nlalloc(double rc)
{
  NeighborList *new;
  new = mymalloc(sizeof(NeighborList));
  new->rc = rc;
  new->size = new->n = 0;
  new->full = 1;		/* include all by default */
  new->list = NULL;
  new->mrkr = NULL;
  return new;
}

/*==========================================================================
 * Retrieve information from a neighbor list.
 *
 * David Deaven 23 February 1994 deaven@ishmael.ameslab.gov 
 *========================================================================*/
void getnl(Geometry *g, int i, nl_t list, int *jp, double *dij,
	double dxij[3], int n[3])
{
  int m, j;
  double x[3], *b=g->box;
  nl_t jl = getnlindex(list);
  j = jl;
  n[0]=getnlx(list);
  n[1]=getnly(list);
  n[2]=getnlz(list);
  for(m=0; m<3; ++m) x[m] = g->x[i*3+m] - g->x[j*3+m] - n[m];
  for(m=0; m<3; ++m) dxij[m] = -(x[0]*b[m*3]+x[1]*b[m*3+1]+x[2]*b[m*3+2]);
  (*dij) = sqrt(dxij[0]*dxij[0] + dxij[1]*dxij[1] + dxij[2]*dxij[2]);
  if((*dij) > 0.0) for(m=0; m<3; ++m) dxij[m] /= (*dij);
  (*jp) = j;
}

