/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: random.c,v 1.4 1996/02/25 19:32:46 deaven Exp $
 * $Author: deaven $
 */
#include "random.h"

#define MBIG 1000000000
#define MSEED 161803398
#define MZ 0
#define FAC (1.0/MBIG)

int ran3_seed = -868;

static struct ran3_state state = { 0, /* iff */ };

void get_ran3_state(struct ran3_state *buf)
{
	*buf = state;
}

void set_ran3_state(struct ran3_state *buf)
{
	state = *buf;
}

float ran3(void)
{
	long mj,mk;
	int i,ii,k;

	if (ran3_seed < 0 || state.iff == 0) {
		state.iff=1;
		mj=MSEED-(ran3_seed < 0 ? -ran3_seed : ran3_seed);
		mj %= MBIG;
		state.ma[55]=mj;
		mk=1;
		for (i=1;i<=54;i++) {
			ii=(21*i) % 55;
			state.ma[ii]=mk;
			mk=mj-mk;
			if (mk < MZ) mk += MBIG;
			mj=state.ma[ii];
		}
		for (k=1;k<=4;k++)
			for (i=1;i<=55;i++) {
				state.ma[i] -= state.ma[1+(i+30) % 55];
				if (state.ma[i] < MZ) state.ma[i] += MBIG;
			}
		state.inext=0;
		state.inextp=31;
		ran3_seed=1;
	}
	if (++state.inext == 56) state.inext=1;
	if (++state.inextp == 56) state.inextp=1;
	mj=state.ma[state.inext]-state.ma[state.inextp];
	if (mj < MZ) mj += MBIG;
	state.ma[state.inext]=mj;
	return mj*FAC;
}

#undef MBIG
#undef MSEED
#undef MZ
#undef FAC

/* Generate a random integer in the range [0,n-1].
 */
int iran3(int n)
{
	register float r = ran3() * n;
	register int i = r;
	if(i >= n) i = n - 1;
	return i;
}
