/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: spline.h,v 1.2 1995/08/29 22:45:15 deaven Exp deaven $
 * $Author: deaven $
 */

#ifndef _SPLINE_H_
#define _SPLINE_H_

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

#define SPL_NATURAL 1.e31
#define SPL_LINEAR  2.e31

#define NATURAL SPL_NATURAL /* to be phased out */

/* 1D cubic spline */
typedef struct spline_s Spline1D;
struct spline_s {
  int n;
  double *x;
  double *y;
  double *d;	/* second derivatives */
};

/* 2d: collect n 1d splines together */
typedef struct spline2di_s ISpline2D;
struct spline2di_s {
  int n;
  double *x;
  Spline1D *s;
};

/* 2d: collect n 1d splines together */
struct spline2d_s {
  int n;
  double *x;
  Spline1D **s;
};
typedef struct spline2d_s Spline2D;

/* prototypes */
void freeSpline1D(Spline1D *old);
Spline1D *newSpline1D(int n);
Spline1D *resizeSpline1D(Spline1D *s, int n);
void   spline(Spline1D *s, double yp1, double ypn);
double splint(Spline1D *s, double x, double *dy);

void   ispline2(ISpline2D *s, double yp1, double ypn);
double isplint2(ISpline2D *s, double x1, double x2, double *dx1, double *dx2);

Spline2D *newSpline2D(int n);
void freeSpline2D(Spline2D *s);
void spline2(Spline2D *s, double yp1, double ypn);
double splint2(Spline2D *s, double x1, double x2, double *dx1, double *dx2);

unsigned char *PackSpline1D(Spline1D *s, unsigned long *size);
Spline1D *UnPackSpline1D(Spline1D *s, unsigned char *b, unsigned long *size);

void WriteCSourceForSpline1D(FILE *f, Spline1D *s, char *name, int IsStatic);
void WriteCSourceForISpline2D(FILE *f, ISpline2D *s, char *name, int IsStatic);
void WriteCSourceForSpline2D(FILE *f, Spline2D *s, char *name, int IsStatic);

Spline1D *ReadSpline1D(FILE *f);
int WriteSpline1D(FILE *f, Spline1D *s);
Spline2D *ReadSpline2D(FILE *f);
int WriteSpline2D(FILE *f, Spline2D *s);

Spline1D *Spline1DFromFunction(double x0, double x1, int n,
  double (*fun)(double));
Spline1D *Spline1DFromSpline(Spline1D *s, int n);
Spline2D *Spline2DFromSpline(Spline2D *s, int n);

#ifdef __cplusplus
}
#endif

#endif /* _SPLINE_H_ */
