/* Two-dimensional splines
 * This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: spline2.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include <stdlib.h>
#include "spline.h"
#include "memory.h"

/*==========================================================================
 * Make 2d spline/interpolation 
 *
 * David Deaven 23 February 1994 deaven@ishmael.ameslab.gov 
 *========================================================================*/
void ispline2(ISpline2D *s, double yp1, double ypn)
{
  int i;
  for(i=0; i<(s->n); ++i)spline(&s->s[i], yp1, ypn);
}

/*==========================================================================
 * Evaluate 2d spline/interpolation
 * x1 = interpolated, x2 = 1d spline
 *
 * x2 is adjusted to fit in spline range, if necessary.
 *
 * David Deaven 23 February 1994 deaven@ishmael.ameslab.gov 
 *========================================================================*/
double isplint2(ISpline2D *s, double x1, double x2, double *dx1, double *dx2)
{
  int k, klo=0, khi=s->n-1;
  double h, a, b, ylo, yhi, dylo, dyhi, ry;
  while(khi-klo > 1) {
    k=(khi+klo)/2;
    if(s->x[k] > x1)khi=k;
    else klo=k;
  }
  h=(s->x[khi])-(s->x[klo]);
  a=((s->x[khi])-x1)/h;
  b=(x1-(s->x[klo]))/h;
  if(x2 > s->s->x[s->s->n - 1]) x2 = s->s->x[s->s->n - 1];
  if(x2 < s->s->x[0]) x2 = s->s->x[0];
  ylo=splint(&(s->s[klo]), x2, &dylo);
  yhi=splint(&(s->s[khi]), x2, &dyhi);
  ry = a*ylo + b*yhi;
  *dx1 = (yhi-ylo)/h;
  *dx2 = a*dylo+b*dyhi;
  return ry;
}

