/* Spline pack-unpack for node communications
 * This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: spline3.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "spline.h"
#include "memory.h"

static void putspline(Spline1D *s, unsigned char *b)
{
	memcpy(b, &(s->n), sizeof(int));
	memcpy(b + sizeof(int),
	  s->x, s->n * sizeof(double));
	memcpy(b + sizeof(int) + s->n * sizeof(double),
	  s->y, s->n * sizeof(double));
	memcpy(b + sizeof(int) + 2 * s->n * sizeof(double),
	  s->d, s->n * sizeof(double));
}

/* Pack up spline into buffer, return.
 */
unsigned char *PackSpline1D(Spline1D *s, unsigned long *size)
{
	unsigned char *b;
	*size = sizeof(int) + s->n * 3 * sizeof(double);
	b = mymalloc(*size);
	putspline(s, b);
	return b;
}

Spline1D *UnPackSpline1D(Spline1D *s, unsigned char *b, unsigned long *size)
{
	int n;
	unsigned long check_size;

	memcpy(b, &n, sizeof(int));
	check_size = sizeof(int) + n * 3 * sizeof(double);
	if(check_size != *size) return NULL;

	s = resizeSpline1D(s, n);

	memcpy(s->x, b + sizeof(int), n * sizeof(double));
	memcpy(s->y, b + sizeof(int) + n * sizeof(double), n * sizeof(double));
	memcpy(s->d, b + sizeof(int) + 2 * n * sizeof(double), n * sizeof(double));

	return s;
}

