/* Write splines as C source code
 * This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: spline4.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include "spline.h"

static void putarray(FILE *f, int n, double *a, char *name, char *suffix)
{
	int i;
	fprintf(f, "static double %s_%s[%d] = {\n", name, suffix, n);
	for(i=0; i<n; ++i)
	  fprintf(f, "%.16g,\n", a[i]);
	fprintf(f, "};\n");
}

void WriteCSourceForSpline1D(FILE *f, Spline1D *s, char *name, int IsStatic)
{
	putarray(f, s->n, s->x, name, "x");
	putarray(f, s->n, s->y, name, "y");
	putarray(f, s->n, s->d, name, "d");

	fprintf(f, "%sSpline1D %s = {%d, %s_%s, %s_%s, %s_%s};\n",
	  IsStatic ? "static " : "",
	  name, s->n, name, "x", name, "y", name, "d");
}

void WriteCSourceForISpline2D(FILE *f, ISpline2D *s, char *name, int IsStatic)
{
	int i;
	char buf[128];

	putarray(f, s->n, s->x, name, "x");
	for(i=0; i<s->n; ++i) {
	  sprintf(buf, "%s_%d", name, i);
	  WriteCSourceForSpline1D(f, s->s + i, buf, 1);
	}

	fprintf(f, "static Spline1D %s_all[%d] = {\n", name, s->n);
	for(i=0; i<s->n; ++i)
	  fprintf(f, "&%s_%d,\n", name, i);
	fprintf(f, "};\n");

	fprintf(f, "%sISpline2D %s = {%d, %s_x, &%s_all};\n",
	  IsStatic ? "static " : "",
	  name, s->n, name, name);
}

void WriteCSourceForSpline2D(FILE *f, Spline2D *s, char *name, int IsStatic)
{
	int i;
	char buf[128];

	putarray(f, s->n, s->x, name, "x");
	for(i=0; i<s->n; ++i) {
	  sprintf(buf, "%s_%d", name, i);
	  WriteCSourceForSpline1D(f, s->s[i], buf, 1);
	}

	fprintf(f, "static Spline1D *%s_all[%d] = {\n", name, s->n);
	for(i=0; i<s->n; ++i)
	  fprintf(f, "&%s_%d,\n", name, i);
	fprintf(f, "};\n");

	fprintf(f, "%sSpline2D %s = {%d, %s_x, %s_all};\n",
	  IsStatic ? "static " : "",
	  name, s->n, name, name);
}
