/* Routines for constructing cubic splines
 * This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: spline5.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include <stdlib.h>
#include "spline.h"
#include "memory.h"

static Spline1D *lspl;
static double lfun(double x)
{
	double dy;
	return splint(lspl, x, &dy);
}

Spline1D *Spline1DFromFunction(double x0, double x1, int n, 
  double (*fun)(double))
{
	int i;
	double dx = (x1 - x0) / (n > 1 ? n - 1 : 1);
	Spline1D *new = newSpline1D(n);
	for(i=0; i<n; ++i) {
	  new->x[i] = x0 + dx * i;
	  new->y[i] = fun(new->x[i]);
	  new->d[i] = 0.0;
	}

	return new;
}

Spline1D *Spline1DFromSpline(Spline1D *s, int n)
{
	lspl = s;
	return Spline1DFromFunction(s->x[0], s->x[s->n - 1], n, lfun);
}

Spline2D *Spline2DFromSpline(Spline2D *s, int n)
{
	int i;
	Spline2D *new = newSpline2D(s->n);

	for(i=0; i<s->n; ++i) {
	  new->x[i] = s->x[i];
	  new->s[i] = Spline1DFromSpline(s->s[i], n);
	}

	return new;
}
