/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: splinefit.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "spline.h"
#include "memory.h"

static char *usage = "usage: %s [-n n] [-o outfn]\n";
static struct spline_s spl;

#define CHUNK 256
void main(int argc, char **argv)
{
  int i, n, nd, nwrite=100;
  double rx, ry, dy;
  FILE *f = stdin, *fout = stdout;

  for(i=1; i<argc; ++i) {
    if(argv[i][0]!='-') { /* assume it's a new filename */
      if((f=fopen(argv[i], "r"))==NULL) {
	fprintf(stderr, "cannot open %s\n", argv[i]);
	exit(1);
      }
    } else switch(argv[i][1]) {
      case 'n': sscanf(argv[++i], "%d", &nwrite); break;
      case 'o': 
	if((fout=fopen(argv[++i], "r"))==NULL) {
	  fprintf(stderr, "cannot open %s\n", argv[i]);
	  exit(1);
	}
        break;
      default:
	fprintf(stderr, "unknown option %c\n", argv[i][1]);
      case 'h':
	fprintf(stderr, usage, argv[0]);
	exit(1);
    }
  }

  n = 0;
  nd = CHUNK;
  spl.x = dmalloc(nd);
  spl.y = dmalloc(nd);

  while(scanf("%lf %lf", &rx, &ry)==2) {
    if(n >= nd) {
      spl.x = realloc(spl.x, (nd + CHUNK)*sizeof(double));
      spl.y = realloc(spl.y, (nd + CHUNK)*sizeof(double));
      nd += CHUNK;
    }
    spl.x[n] = rx; spl.y[n] = ry;
    ++n;
  }
  fprintf(stderr, "read %d data, writing %d splined points\n", n, nwrite);

  spl.n = n;
  spl.d = dmalloc(n);
  spline(&spl, NATURAL, NATURAL);

  fprintf(fout, "3 #1=(x) #2=(y) #3=(dy/dx)\n");
  for(i=0; i<nwrite; ++i) {
    rx = spl.x[0] + ((double) i) * (spl.x[n-1] - spl.x[0]) / ((double) nwrite-1);
    ry = splint(&spl, rx, &dy);
    fprintf(fout, "%e %e %e\n", rx, ry, dy);
  }
  if(fout != stdout) fclose(fout);

  exit(0);
}
