/* Routines for evaluating cubic splines
 * This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: splint.c,v 1.2 1995/08/29 22:45:15 deaven Exp $
 * $Author: deaven $
 */

#include <math.h>
#include <stdlib.h>
#include "spline.h"

double splint(Spline1D *s, double x, double *dy)
{
  register int k, klo = 0, khi = (s->n)-1;
  double h, a, b, a3, b3, ry;

  while(khi-klo > 1) {
    k = (khi + klo)/2;
    if(s->x[k] > x)
      khi = k;
    else
      klo = k;
  }

  h = (s->x[khi]) - (s->x[klo]);
  a = ((s->x[khi])-x)/h; a3 = a*a*a;
  b = (x-(s->x[klo]))/h; b3 = b*b*b;

  ry = a*(s->y[klo])+b*(s->y[khi]) 
	+ ((a3-a)*(s->d[klo])+(b3-b)*(s->d[khi]))*(h*h)/6.0;
  *dy = ((s->y[khi])-(s->y[klo]))/h 
	+ ((3.0*b*b-1.0)*(s->d[khi])-(3.0*a*a-1.0)*(s->d[klo]))*h/6.0;
  return ry;
}

