/* GENETIC ALGORITHM AND RELATED CODE
 * Copyright (c) 1995 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: temperature.c,v 1.4 1996/03/28 20:03:51 deaven Exp $
 * $Author: deaven $
 */
#include <stdio.h>
#include <math.h>
#include "random.h"
#include "tools.h"

/* Evaluate the kinetic energy for n particles in
 * the given dimension (e.g. dimension=3 for 3 coordinates per mass).
 * Use unit masses if m[] is NULL.
 */
double GetKinetic(double *v, double *m, int dimension, int n)
{
	int i, j, k = 0;
	double ek = 0.0;
	for(i=0; i<n; ++i)
	{
	  for(j=0; j<dimension; ++j)
	  {
	    ek += (m==NULL ? 1.0 : m[i]) * v[k] * v[k];
	    k++;
	  }
	}
	return (0.5 * ek);
}

/* Set the kinetic energy of n particles in
 * the given dimension (e.g. dimension=3 for 3 coordinates per mass).
 * Use unit masses if m[] is NULL.
 * Return the original kinetic energy.
 */
double SetKinetic(double *v, double *m, int dimension, int n, double target)
{
	int i;
	double ek, ek0 = GetKinetic(v, m, dimension, n);
	double ekmin = 1.e-20 / n;

	ek = ek0;
	while(ek < ekmin) {
	  /* Reset to random velocities, since they are too small.
	   */
	  for(i=0; i<(n * dimension); ++i) v[i] = ran3() - 0.5;
	  ek = GetKinetic(v, m, dimension, n);
	}
	ek = sqrt((target > 0.0 ? target : 0.0) / ek);
	for(i=0; i<(n * dimension); ++i) v[i] *= ek;

	return ek0;
}

