/* This file is part of David Deaven's personal library.
 * Copyright (c) 1994 David Deaven
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: tools.h,v 1.24 1996/08/08 19:48:17 deaven Exp deaven $
 * $Author: deaven $
 */

#ifndef _TOOLS_H_
#define _TOOLS_H_

#include <stdio.h>
#include <math.h>

#ifdef __cplusplus
extern "C" {
#endif

#if defined(__BSD__) || defined(__NetBSD__)
#define DMD_TOOLS_NEED_SETENV	0
#define DMD_TOOLS_NEED_STRSEP	0
#else
#define DMD_TOOLS_NEED_SETENV	1
int setenv(char *var, char *value, int overwrite);
#define DMD_TOOLS_NEED_STRSEP	1
char *strsep(char **stringp, char *delim);
#endif

struct performance_s {
	unsigned long kflops;	/* single cpu speed */
};

struct dmd_iovec {
	unsigned char *iov_base;
	unsigned long iov_len;
};

double *mmul3(double a[3*3], double x[3], double y[3]);
double *mtrn3(double a[3*3], double x[3]);
double *vmmul3(double a[3*3], double *x, double *y, int nvec);
double *mtmul3(double a[3*3], double x[3], double y[3]);
double *mttrn3(double a[3*3], double x[3]);
double *vmtmul3(double a[3*3], double *x, double *y, int nvec);

double *matmul3(double a[3*3], double b[3*3], double c[3*3]);
void matinv3(double ai[3*3], double a[3*3], double *det);

int dmd_readv(int sock, struct dmd_iovec *iov, int iovcnt);
int dmd_writev(int sock, struct dmd_iovec *iov, int iovcnt);

void performance(struct performance_s *perf);

char *findfile(char *name, char *pathspec);

char **getargv(char *s, int where, int *argc, char **argv);
char **fgetargv(FILE *f, int where, int *argc, char **argv);

FILE *fopen_path(char *pathlist, char *filename, char *mode);
FILE *fopen_with_default(char *filename, char *mode, FILE *def);

void PrintRMp(FILE *f, double **a, int n, int m, char *s);
void PrintRM(FILE *f, double *a, int n, int m, char *s);
void PrintRMPart(FILE *f, double *a, int n, int m, char *s, 
	int nlo, int nhi, int mlo, int mhi);
void PrintCMp(FILE *f, double **ar, double **ai, int n, int m, char *s);
void PrintCM(FILE *f, double *ar, double *ai, int n, int m, char *s);
void PrintCMPart(FILE *f, double *ar, double *ai, int n, int m, char *s, 
	int nlo, int nhi, int mlo, int mhi);

int NumericalDerivative(double (*fun)(double, void *), void *res,
  double x, double f, double tol, double *dfdx);

int conjgrad(double x[], double f[], double *fret, int n,
  double ftol, int iter,
  int fun(double *, double *, double *, int), FILE *);
int conjgrad_engine(double x[], double f[], double *fret, int n,
  double ftol, int iter, int fletcher, double lintol, int liniter,
  int (*fun)(double *, double *, double *, int), FILE *fout);
int mnbrak(double *ax, double *bx, double *cx,
  double *fa, double *fb, double *fc,
  int (*fun)(double x, double *y, double *dfun, void *fenv), void *fenv);
int linmin(double *p, double *xi, int n, double tol, int iter,
  int (*fun)(double *, double *, double *, int), double *fret, double *f);
int dbrent(double ax, double bx, double cx, double fb, double dfb,
	int (*fun)(double x, double *y, double *df, void *fenv),
	void *fenv, double tol, int iter, double *xmin, double *fret);

/* This macro is depreciated in favor of using rint(), ceil(), or
 * floor() directly.
 */
#define anint(x) rint(x)

int optdfp_engine(double x[], double f[], double *ep, int n,
  double frceps, double eps, double step0, double dotmin, int maxloop,
  int use_bfgs, int itmax, int irmax, double *hmat_passed, 
  void *fun_resources,
  void (*callback)(double *, double *, double *, int, void *),
  int (*fun)(double *, double *, double *, int, void *), FILE *fout);
int optdfp(double x[], double f[], double *ep, int n,
  double frceps, double eps,
  int itmax, int irmax, double hmat[],
  void *fun_resources,
  int (*fun)(double *, double *, double *, int, void *), FILE *fout);
int optbfgs(double x[], double f[], double *ep, int n,
  double frceps, double eps,
  int itmax, int irmax, double hmat[], 
  void *fun_resources,
  int (*fun)(double *, double *, double *, int, void *), FILE *fout);

int Marquardt(double x[], double y[], double sig[], int ndata,
	double a[], int ia[], int ma,
        void (*fun)(double, double *,double *,double *,int),
	double *covar, double *alpha, double *chisq, double *alamda);

int gaussj(double *a, int n, double *b, int m);

int amoeba(double **p, double *y, int ndim, double ftol, int nmax,
	int (*funk)(double *, double *, void *), void *res, int *nfunk);

void lstsquare(double x[], double y[], int nd, 
	double *coef, int order, double *err, double *rmserr, double *s);

void calcfermi(double *chem, double *selec, double focc[], double eigen[],
	double weight[], double telec, int mdimen, int ns);

void *new_nordsk(int n);
void delete_nordsk(void *res);
int nordsk(void *res, int dimension, int n, double *hp,
	double x[], double v[], double f[], double *e, double *am,
	int fun(double *, double *, double *, int));

double GetKinetic(double *v, double *m, int dimension, int n);
double SetKinetic(double *v, double *m, int dimension, int n, double target);

int f77read(void *ptr, int size, int nobj, FILE *stream);
int f77write(void *ptr, int size, int nobj, FILE *stream);

void Cross(double u[3], double v[3], double w[3]);
double Norm(double u[3]);
void Gentra(double xcam[3], double vcam[3], double up[3],
  double t[4*4], double scam[3*3]);
void Transf(double x[3], double t[4*4], double z[3]);
void Transfvec(double x[3], double t[4*4], double z[3]);

double *YPRRotationMatrix(double yaw, double pitch, double roll, double t[3*3]);
double YPRIntegrate(int n, double *ypr, double *w, double *I,
	double h, double *torque);
double *YPRTransformDerivative(double xi[3], double t[3*3]);
double *YPRtoQ(double ypr[3], double q[4]);
double *YPRfromR(double R[3*3], double ypr[3]);

double *QRotationMatrix(double q[4], double t[3*3]);
void *QNewIntegrateContext(int n);
void QDeleteIntegrationContext(void *context);
double QIntegrate(void *context,
  double *q, double *w, double *I, double h, double *t);
double QIntegrateSimpson(void *context,
  double *q, double *w, double *I, double h, double *t);
double *QtoYPR(double q[4], double ypr[3]);

/* Error handling codes
 */
enum dmd_errors {
	DMD_NORMAL = 0,
	DMD_SYSTEM,		/* system error */
	DMD_INVALID_ARGUMENT,	/* invalid argumnets to function */
	DMD_COMPONENT_FAILURE,	/* auxillary software failed */
	DMD_RESOURCE_UNAVAILABLE, /* resource unavailable */
	DMD_INVALID_INPUT,	/* input is invalid */
	DMD_MAXERROR
};

extern int dmd_errno;	/* errno, from above */
int dmd_perror(char *s);
void dmd_fatal_error(char *s);
int dmd_system_error(char *s);

#ifdef __cplusplus
}
#endif

#endif /* _TOOLS_H_ */
